import os
import time
import csv
import pandas as pd
import undetected_chromedriver as uc
from bs4 import BeautifulSoup

OUTPUT_FILE = "minfin_books.csv"
START_URL = "https://minfin.gov.ru/ru/ministry/historylib/"
WAIT_TIME = 3  # секунд между циклами подгрузки

def load_existing_links():
    if not os.path.exists(OUTPUT_FILE):
        return set()
    df = pd.read_csv(OUTPUT_FILE)
    return set(df['source_link'].values)

def init_driver():
    options = uc.ChromeOptions()
    options.add_argument('--headless=new')
    options.add_argument('--no-sandbox')
    options.add_argument('--disable-dev-shm-usage')
    options.add_argument('--window-size=1920,1080')
    options.add_argument("--user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
                         "AppleWebKit/537.36 (KHTML, like Gecko) "
                         "Chrome/125.0.0.0 Safari/537.36")
    return uc.Chrome(options=options)

def scroll_to_bottom(driver):
    last_height = driver.execute_script("return document.body.scrollHeight")
    while True:
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        time.sleep(WAIT_TIME)
        new_height = driver.execute_script("return document.body.scrollHeight")
        if new_height == last_height:
            break
        last_height = new_height

def extract_books(driver, existing_links):
    soup = BeautifulSoup(driver.page_source, "html.parser")
    tiles = soup.select(".history-tile")
    books = []
    for tile in tiles:
        try:
            title_tag = tile.select_one(".document-title a")
            if not title_tag:
                continue
            name = title_tag.get_text(strip=True)
            source_link = "https://minfin.gov.ru" + title_tag["href"]
            if source_link in existing_links:
                continue
            pdf_btn = tile.select_one('a[title*="Скачать"]')
            if pdf_btn and pdf_btn.get("href"):
                pdf_link = "https://minfin.gov.ru" + pdf_btn["href"]
            else:
                continue
            books.append({
                "name": name,
                "source_link": source_link,
                "book_link": pdf_link
            })
        except Exception as ex:
            print(f"❌ Ошибка извлечения: {ex}")
    return books

def append_to_csv(book_data):
    file_exists = os.path.exists(OUTPUT_FILE)
    with open(OUTPUT_FILE, "a", newline="", encoding="utf-8") as f:
        writer = csv.DictWriter(f, fieldnames=["name", "source_link", "book_link"])
        if not file_exists:
            writer.writeheader()
        for row in book_data:
            writer.writerow(row)

def main():
    existing_links = load_existing_links()
    print(f"📄 Уже в файле: {len(existing_links)} записей")

    driver = init_driver()
    try:
        driver.get(START_URL)
        scroll_to_bottom(driver)
        print("📦 Загружено всего плиток — прокрутка завершена")
        new_books = extract_books(driver, existing_links)
        print(f"📚 Новых найдено: {len(new_books)}")

        if new_books:
            append_to_csv(new_books)
            print(f"✅ Добавлено в CSV: {len(new_books)}")
        else:
            print("🔍 Новых публикаций не найдено")
    finally:
        driver.quit()

if __name__ == "__main__":
    try:
        main()
    except Exception as ex:
        print(f"💥 Ошибка в исполнении скрипта: {ex}")


